searchNodes=[{"doc":"A simple distributed load test runner.Loadex was created with two things in mind - genarating huge loads in a controlled manner, while being able to fully customize the test's flow. These goals are achieved by using plain Elixir to create scenarios and then laveraging Elixir's massive concurrency capabilities to run them on one or multiple machines.Example:defmodule ExampleScenario do use Loadex.Scenario setup do 1..100 end scenario index do loop_after 500, 10, iteration do IO.puts(&quot;My number is \#{index}, iteration \#{iteration}!&quot;) end end teardown index do IO.puts(&quot;Bye from \#{index}!&quot;) end endFor detailed instructions on how to create a scenario please refer to Loadex.Scenario.","ref":"Loadex.html","title":"Loadex","type":"module"},{"doc":"Adds nodes into the Loadex cluster.","ref":"Loadex.html#join_cluster/1","title":"Loadex.join_cluster/1","type":"function"},{"doc":"Runs scenarios.Running a scenario means executing its setup callback and passing its results to the scenario implementation. For more detailed information on how to create scenarios please refer to Loadex.Scenario.When running in a distributed environment (see join_cluster/1), the setup callback will be executed on a node run/1 is called on and its results will be distributed along the cluster.By default scenarios are loaded from ./scenarios directory and executed all at the same time. A single scenario can be specified by passing a scenario option.Scenarios can be restarted after crashing or quitting by passing restart: true option.Rate (per second), at which scenarios are started can be adjusted by passing a rate option. Note: this doesn't affect restart rate.Example:iex&gt; Loadex.run(scenario: &quot;./scenarios/example_scenario.exs&quot;, rate: 30, restart: true)","ref":"Loadex.html#run/1","title":"Loadex.run/1","type":"function"},{"doc":"Stops all scenarios.","ref":"Loadex.html#stop_all/0","title":"Loadex.stop_all/0","type":"function"},{"doc":"A set of macros used to create Loadex scenarios.What is a scenario?TL;DR: Scenario is basically a load test case.The common problem with load testing is that a synthtetic load doesn't really produce great results. Performance is measured, the application is deployed to production and then it crashes under much smaller load, than we generated using our favourite load testing tool.The reason is that synthetic load is, well, synthetic. While testing a single REST endpoint probably may not be a problem, more complex workflows can run into such issues quite easily. It makes a lot of sense, then, for our load tests to reflect the real world use cases of the application we're testing. This may include aquiring a token for authentication, creating a persistent connection using a required protocol, executing some specific handshake or initialization etc.It requires expressiveness usually associated with programming languages. Scenarios provide a way of describing complex workflows with Elixir code and libraries. They're then executed concurrently to generate substantial loads.Creating a scenarioSetupEach Loadex scenario starts with a setup/1 macro. It defines how many workers need to be started and what data should they receive as an optional seed. This can be done in one of two ways: either returning a Range:setup do 1..10 end...or a Stream of Loadex.Scenario.Spec structs with an unique id and a seed for each scenario:setup do load_users_from_csv() |&gt; Stream.map(fn %User{id: id} = user -&gt; Loadex.Scenario.Spec.new(id, user) end) endScenarioThis is where magic happens. Scenario's code is executed in a separate process for every element returned by the setup. This element, a seed, is given as a prameter to the scenario/2 macro:defmodule ExampleScenario do use Loadex.Scenario setup do load_users_from_csv() |&gt; Stream.map(fn %User{id: id} = user -&gt; Loadex.Scenario.Spec.new(id, user) end) end scenario %User{login: login, password: password} do token = AuthClient.get_token(login, password) loop_after 2000, 10, _repetition do ExternalServiceClient.generate_some_load(token) end end endThis simple scenario above loads a bunch of users from a CSV during the setup stage. Then each user concurrently aquires a token and finally starts making calls, every two seconds and ten in total, to the external service we want to test.Note that we're using the loop_after/4 macro instead of :timer.sleep/1 and Enum.each/2 or a list comprehension. The reason for this is that our scenario is run in a process and iterating on a list (or Range) and :timer.sleep/1 calls are blocking it. Meanwhile, loop_after/4 is asynchronous to ensure the worker can receive and process messages.While it may not be an issue in your case, it is strongly advised to use built-in helpers to ensure all the performance benefits, that using Elixir and OTP gives us. Please refer to their documentation for more details.TeardownIf there's any setup you'd like to undo after your scenario finishes, teardown/2 is a place to do it:defmodule ExampleScenario do use Loadex.Scenario setup do load_users_from_csv() |&gt; Stream.map(fn %User{id: id} = user -&gt; ExternalServiceClient.create_account(user) Loadex.Scenario.Spec.new(id, user) end) end scenario %User{login: login, password: password} do # do stuff... end teardown %User{} = user do ExternalServiceClient.delete_account(user) end end","ref":"Loadex.Scenario.html","title":"Loadex.Scenario","type":"module"},{"doc":"Terminates the scenario. teardown/2 will be executed after this call.","ref":"Loadex.Scenario.html#end_scenario/0","title":"Loadex.Scenario.end_scenario/0","type":"macro"},{"doc":"A helper for creating an asynchronous, non-blocking loops using message-passing.loop 10, iteration do IO.puts(&quot;\#{iteration}&quot;) endParams:iterations - how many times should the code in the do block be executedhibernate_or_standby - (optional) allows you to hibernate the underlying GenServer between each pass. Defaults to :standbymatch - a match pattern. Currently only an iteration number is passed here","ref":"Loadex.Scenario.html#loop/4","title":"Loadex.Scenario.loop/4","type":"macro"},{"doc":"A helper for creating an asynchronous, non-blocking loops using message-passing.loop_after 100, 10, iteration do IO.puts(&quot;\#{iteration}&quot;) endParams:time - the delay beteewn each passiterations - how many times should the code in the do block be executedhibernate_or_standby - (optional) allows you to hibernate the underlying GenServer between each pass. Defaults to :standbymatch - a match pattern. Currently only an iteration number is passed here","ref":"Loadex.Scenario.html#loop_after/5","title":"Loadex.Scenario.loop_after/5","type":"macro"},{"doc":"Scenario's implementation.A single seed element returned from setup/2 is passed as an argument.As code inside this macro will be executed inside a concurrent process, using helpers provided by this module is strongly advised for operations such as loops, to prevent the process from blocking.","ref":"Loadex.Scenario.html#scenario/2","title":"Loadex.Scenario.scenario/2","type":"macro"},{"doc":"Sets up the scenario.Must return a Range or a list of Loadex.Scenario.Spec structs. Each value will be passed as a seed to a separate process running the scenario.This callback is executed in by the runner, before any scenario starts.","ref":"Loadex.Scenario.html#setup/1","title":"Loadex.Scenario.setup/1","type":"macro"},{"doc":"Cleans up after a scenario.Is given a seed from setup/2 as a parameter.This callback is executed by each individual scenario worker.","ref":"Loadex.Scenario.html#teardown/2","title":"Loadex.Scenario.teardown/2","type":"macro"},{"doc":"Allows user to act upon receiving a specific message.wait_for {:msg, message} do IO.puts(&quot;{message}&quot;) endWhile this macro has blocking semantics, in a sense it will execute blocks for consecutive calls one after another, it doesn't actually block the process. This means any code placed after wait_for/2 will be executed immediately. For blocking behaviour, use receive/1.Params:match - a match pattern for a specific message","ref":"Loadex.Scenario.html#wait_for/2","title":"Loadex.Scenario.wait_for/2","type":"macro"},{"doc":"LoadexA simple distributed load test runner.Loadex was created with two things in mind - genarating huge loads in a controlled manner, while being able to fully customize the test's flow. These goals are achieved by using plain Elixir to create scenarios and then laveraging Elixir's massive concurrency capabilities to run them on one or multiple machines.Docs can be found at https://hexdocs.pm/loadex.","ref":"readme.html","title":"Loadex","type":"extras"},{"doc":"Add loadex to your list of dependencies in mix.exs:def deps do [ {:loadex, &quot;~&gt; 0.1.0&quot;} ] endExample:defmodule ExampleScenario do use Loadex.Scenario setup do 1..100 end scenario index do loop_after 500, 10, iteration do IO.puts(&quot;My number is \\\#{index}, iteration \\\#{iteration}!&quot;) end end teardown index do IO.puts(&quot;Bye from \\\#{index}!&quot;) end endThen:iex&gt; Loadex.run(scenario: &quot;./scenarios/example_scenario.exs&quot;, rate: 30, restart: true)More information can be found in the docs","ref":"readme.html#installation","title":"Loadex - Installation","type":"extras"}]